;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4CC2			;196450

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@Laddr=09D8			;Line address subroutine
@Vars=4010			;Start of vars area
@DFile=400C			;Address of display file
@NxtLine=4029			;Next program line to be executed
@LastK=4025			;Keyboard debounce value
@LoopFlag=4083			;16515, Player stopped flag
@MainForm=49A1			;Stored mainform address
@WorkArea=BF9A			;49050, work area,102 bytes

;================================
; Jump Table
;================================
	JP KeyLoop		;19650 Key loop and sorter
	JR LoadScr		;19653 Load main form
	JR SaveScr		;19655 Save main form
	JR Release		;19657 Test for keypress
	00 00 00		;19659 Spare
	JP FileDrop		;19662 File dropdown menu
	JP TuneDrop		;19665 Tunes dropdown menu
	JP WallDrop		;19668 Walls dropdown menu
	JP HelpDrop		;19671 Help dropdown menu
	JR Stopped		;19674 Test "Stopped" flag
	JP PrntInfo		;19676 Print PT3 information
@WPage
	00			;19679, Walls dropdown page number
@TPage
	00			;19682, Tunes dropdown page number
	00			;19683, Spare
;================================
; Load screen
;================================
@LoadScr
	PUSH HL
	LD HL,MainForm		;Set source
	LD DE,(DFile)		;Set destination
	JR +7
;================================
; Save screen
;================================
@SaveScr
	PUSH HL
	LD HL,(DFile)		;Set source
	LD DE,MainForm		;Set destination
	LD BC,0319		;BC=byte count
	LDIR			;Copy source to desination
	POP HL
	RET			;Return 0
;================================
; Wait for key release
;================================
@Release
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Release		;Yes, loop back
	RET
;================================
; Test "Stopped" value and keypress
; Display wallpaper until tune stopped
; or keypressed.
;================================
@Stopped
	CALL Release		;Wait for no key press
	LD HL,LoopFlag		;Loopflag addr
	BIT 7,(HL)		;Test stopped bit.
	JR Z +6			;If 0 move on, still playing
	RES 7,(HL)		;Tune ended, reset stopped flag
	BIT 0,(HL)		;Test loop flag
	JR NZ +9		;If NZ don't loop
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ +2		;Yes, return
	JR Stopped+3		;No, quit if keypressed
	CALL Release		;Test for key press
	CALL LoadScr		;Restore main screen
	LD C,5A			;BC=90
	RET			;GOTO 90
;================================
; Print dropdown menu
;================================
@DropDown
	CALL SaveScr		;Save main screen
	LD HL,(NxtLine)
@DropMenu
	LD BC,0005		;Skip REM header
	ADD HL,BC		;Start of data
	CALL Release		;Wait for key release
	LD B,(HL)		;Get Ycoord
	INC HL
	LD C,(HL)		;Get Xcoord
	INC HL			;Start of print data
	PUSH HL			;PUSH datapointer
	CALL PrntPos		;Get print position of the NW corner of dropdown
	POP BC			;POP Datapointer into BC
@DropY
	LD (Drop+1),HL		;Save start of row addr
@DropLoop
	LD A,(BC)		;Get chr to print
	CP 76			;EO REM?
	RET Z			;Return printing done
	CP 0C			;EOL marker ()?
	JR Z Drop		;Jump if EOL
	LD (HL),A		;Print A
	JR +7
@Drop
	LD HL,0000		;Get start of row addr
	ADD HL,DE		;Next row addr
	INC BC			;Increase data pointer
	JR DropY		;Start of row addr
	INC HL			;Increase print position
	INC BC			;Increase data pointer
	JR DropLoop
;================================
; File menu dropdown
;================================
@FileDrop
	CALL DropDown		;Print file dropdown menu
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	LD HL,FileKeys		;Valid keys table
	LD BC,0010 		;Byte count
	CPIR 			;Locate key
	JR NZ +1		;Jump if no match
	INC B			;B=Hi byte of line number
@SetLine
	LD C,(HL)
	LD H,B
	LD L,C
	CALL Release
	CALL Laddr
	LD (NxtLine),HL
	JP Loadscr
;================================
@FileKeys			;Key code/lo byte of line number
	"P" 90			;400, Play/Pause
	"B" 40			;320, Back
	"N" 4A			;330, Next
	"L" 54			;340, Loop
	"S" 5E			;350, Shuffle
	"I" 68			;360, PT3 Info
	"M" 7C			;380, PAL Margin
	"C" 86			;390, Splash Screen
	5A			;EOTable marker=GOTO 90
;================================
; Tunes dropdown
;================================
@TuneDrop
	CALL DropDown		;Print tunes dropdown
	LD H,01 		;Hi byte of REM line
	LD A,(TPage)		;Get current page number
	ADD A,58		;Adjust page number 
	LD L,A			;Lo byte of REM line, HL=501,2,3 or 4
	CALL LAddr		;Get line address
	CALL DropMenu		;Print dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CP "A"			;Key<"A"?
	JR C WrongKey		;Jump if key<"A"
	CP "W"			;Key>="W"?
	JR NC WrongKey		;Jump if key>"V"
	SUB 25 			;A=0 to 22
	LD D,A 			;Save A			
	CALL Release		;Wait for key release
	LD A,D 			;Restore A
	CP 16			;A="V"?
	JR NZ +14		;Jump if not "V"
	LD HL,(Vars)		;Locate page number
	LD BC,000E 
	SBC HL,BC 
	LD A,(HL) 		;Get page number
	LD (TPage),A 		;Set page number
	JR TuneDrop+3		;Print next page
	LD A,(Tpage) 		;Get page number
	SUB 9D			;A=0,1,2,3,4...
	AND A			;A=0?
	LD B,A			;Set DJNZ counter
	LD A,H			;A=0, Z flag not changed
	JR Z +4			;Jump if A was 0
	ADD A,15		;Add offset to A (A=A+21)
	DJNZ -4			;Loop until B=0
	ADD A,D			;Add item number to offset
	LD C,A 			;BC=Tune number
	PUSH BC 		;PUSH tune number
	CALL LoadScr		;Restore screen
	POP BC 			;POP tune number
	RET			;Return tune number to BASIC
;==========================
; Walls dropdown
;================================
@Walldrop
	CALL DropDown		;Print tunes dropdown
	LD H,03 		;Hi byte of REM line
	LD A,(WPage)		;Get current page number 
	SUB 7C 			;Adjust page number
	LD L,A 			;Lo byte of REM line, HL=801,2,3,4,5,6 or 7
	CALL Laddr		;Get line address
	CALL DropMenu		;Print dropdown
	CALL Keyscan		;Start keybpard loop
	JR Z -5			;Loop until keypressed
	LD BC,0000 		;Initialize
	CP "A"			;Key<"A"?
	JR NC +4		;Jump if key>="A"
@WrongKey
	CALL LoadScr		;Restore mainform
	RET			;Return to BASIC
	CP "W"			;Key>="W"?
	JR NC -8		;Jump if Key>"V"
	LD B,24 		;Adust key value by 36
	LD C,0E 		;C=Xcoord=14
	SUB B 			;A=A-36=Row Number 2 to 23
	LD B,A 			;B=Ycoord, C=Xcoord
	CALL PrntPos		;Get Print Position address
	CALL Release		;Wait for key release
	LD A,(HL) 		;Get menu item chr, "gA" to "gV"
	CP BB			;"gV"?
	INC HL
	INC HL			;Start of  menu item
	JR NZ +10		;Jump if not "gV"
	LD DE,0005 		;Locate page number chr
	ADD HL,DE 
	LD A,(HL) 		;Get page number
	LD (WPage),A 		;Set page number
	JR WallDrop+3		;Print next page
	LD B,H 			
	LD C,L 			;Return menu item address to BASIC
	RET
;================================
; Help menu dropdown
;================================
@HelpDrop
	CALL DropDown		;Print help dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CP "H"			;"H"ot keys
	JR NZ KeyK
	LD HL,+630
	JP SetLine+3
@KeyK
	CP "K"			;"K"ey Guide
	JR NZ +2
	JR KeyGuide
	CP "A"			;"A"bout
	JR Z About
@GOTO90
	LD HL,005A		;GOTO 90
	JP SetLine+6
;================================
; Key Guide
;================================
@KeyGuide
	LD HL,+615		;REM 615
	JR +3
;================================
; About
;================================
@About
	LD HL,+620		;REM 620
	CALL LAddr
	LD (NxtLine),HL
	CALL LoadScr		;Reload main screen
	CALL DropDown+3		;Print REM 715 or 720
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CALL Release		;Wait for key release
	CALL LoadScr		;Reload main screen
	JR GOTO90
;================================
; Print Position
;  B=Ycoord
;  C=Xcoord
;  BC unchange
;  On exit HL=Print Position address
;================================
@PrntPos
	PUSH BC			;PUSH YXcoords
	LD HL,0000		;Set accumulator to 0
	LD DE,0021		;Bytes/row
	LD A,B
	AND A			;Row=0?
	JR Z +3			;Yes, move on
	ADD HL,DE		;HL=33*row value
	DJNZ -3
	ADD HL,BC		;HL=33*row value+column offset
	LD BC,(DFile)		;Get DFile
	INC BC			;Skip 1st N/L chr
	ADD HL,BC		;HL=print position
	POP BC			;POP YXcoords
	RET	
;================================
; Test for keypress.
; On exit...
;   A=chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress, D=255 if no key press
	INC D 			;0?
	RET Z
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	INC A			;Set Z flag
	LD A,(HL)
	RET			;Return chr code
;================================
; Key loop
;================================
@KeyLoop
	CALL TrackName		;Track name display
	LD HL,LoopFlag		;Loopflag addr
	BIT 7,(HL)		;Test stopped bit.
	JR Z KeyTest		;Jump if still playing
	RES 7,(HL)		;Reset stopped flag
	BIT 0,(HL)		;Test loop flag
	JR Z KeyLoop		;Jump if loop flag=0
	LD BC,0064		;Start next tune, BC=100
	RET			;GOTO 100
@KeyTest
	CALL KeyScan		;Get key press
	JR Z KeyLoop		;No, loop until key pressed
	CP "F"			;"F"ile dropdown?
	JR NZ +4
	LD BC,+250
	RET			;GOTO 250
	CP "T"			;"T"unes dropdown?
	JR NZ +4
	LD BC,+500
	RET			;GOTO 500
	CP "W"			;"W"alls dropdown?
	JR NZ +7
	CALL SaveScr		;Save screen
	LD BC,+800
	RET			;GOTO 800
	CP "H"			;"H"elp dropdown?
	JR NZ +4
	LD BC,+600
	RET			;GOTO 600
	CP "X"			;E"x"it key?
	JR NZ +4
	LD BC,+700
	RET			;GOTO 700
	LD HL,FileKeys		;Hot keys table
	LD BC,0012 		;Byte count
	CPIR 			;Locate key
	JR NZ  Keyloop		;Loop if none found
	CP "B"			;"B"ack?
	JR Z +2
	CP "N"			;"N"ext
	CALL NZ Release		;Wait for key release except for "B" & "N"
	LD B,01 		;Hi byte line number
	LD C,(HL) 		;Lo byte line number
	RET			;GOTO BC
;================================
; Track Name Display
;================================
@TrackName
	LD HL,(DFile)		;Get DFile
	LD DE,00D0		;Offset to track name
	ADD HL,DE		;HL=Track name location
	LD D,H			;DE=Track name location
	LD E,L
	LD B,0C			;Byte count
;NameLoop
	LD A,(HL)		;Get chr at HL
	ADD A,80		;Add 128
	LD (HL),A		;Set new chr
	INC HL			;Next chr
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR Z +9			;Jump to delay if no keypress
	EX DE,HL		;Set HL to track name location
	LD B,0C			;Byte count
	RES 7,(HL)		;Reset track name
	INC HL
	DJNZ -5
	RET
;Delay
	PUSH BC			;PUSH byte count
	LD BC,0A01		;Delay loop
	DEC BC 
	LD A,B 
	OR C 
	JR NZ -5
	POP BC			;POP byte count
	DJNZ -33		;Jump to NameLoop until B=0
	RET
;================================
;Print Header Info
;================================
@PrntInfo
	LD HL,WorkArea		;Clear the work area
	LD (HL),80		;Fill chr
	LD DE,WorkArea+1	;WorkArea+1
	LD BC,0064		;Byte count, 100
	LDIR			;Clear work area
	LD HL,801E		;LDIR source, PT3 info header
	LD DE,WorkArea		;LDIR destination
	LD C,44			;Byte count,68
	LDIR			;Copy PT3 header to workarea
	LD HL,WorkArea		;Get start of PT3 data
	DEC HL			;PT3 data-1
	LD B,44			;Byte count
@ASC01				;ASCII to ZX81 conversion
	PUSH BC			;PUSH counter
	INC HL
	LD A,(HL)
	CP 41			;Code<A?
	JR C Numeral		;Yes, move on
	CP 41			;Code<A?
	JR C Lower		;Yes, move on
	CP 5B			;Code>Z?
	JR NC Lower		;Yes, move on
	SUB 1B			;Must be an uppercase chr.
	JR Change		;Convert it to ZX81
@Lower
	CP 61			;Code<a?
	JR C Symbol		;Yes, move on
	CP 7B			;Code>z?
	JR NC Symbol		;Yes, move on
	SUB 3B			;Must be a lowercase chr.
	JR Change		;Convert it to ZX81
@Numeral
	CP 30			;Code<0?
	JR C Symbol		;Yes, move on
	CP 3A			;Code>9?
	JR NC Symbol		;Yes, move on
	SUB 14			;Must be a numeral
	JR Change		;Convert it to ZX81
@Symbol
	EX DE,HL		;Save char address
	LD BC,0046		;Table size
	LD HL,Symbolz
	CPIR			;Locate A in table
	LD A,(HL)		;Get the conversion value
	EX DE,HL		;Restore chr address
	JR Z Change		;Unsupported ASCII chr
	LD A,80			;Unsupported ASCII character
@Change
	SET 7,A
	LD (HL),A
	POP BC			;POP counter
	DJNZ ASC01
	LD HL,(DFile)		;Get DFile
	LD BC,0274		;Offset to print position
	ADD HL,BC		;Print position
	LD DE,WorkArea
	EX DE,HL		;Set pointers for LDIR
	LD A,03			;3 lines to print
	LD B,00
@NextLine
	LD C,20			;32 Bytes/line
	LDIR			;Copy line to DFile
	INC DE			;Skip N/L
	INC HL
	DEC A			;Decrease counter
	JR NZ NextLine
	RET
;================================
@Symbolz
	20 00			; Space
	22 0B			; "
	A3 0C			; 
	24 0D			; $
	3A 0E			; :
	3F 0F			; ?
	28 10			; (
	29 11			; )
	3E 12			; >
	3C 13			; <
	3D 14			; =
	2B 15			; +
	2D 16			; -
	2A 17			; *
	2F 18			; /
	3B 19			; ;
	2C 1A			; ,
	2E 1B			; .
;================================
;Replacement symbols
	09 00			; TAB with space		
	7E 16			; ~ with -
	60 0B			; ` with "
	21 1D			; ! with 1
	40 17			; @ with *
	23 15			; # with +
	25 86			; % with NW/SE double pixel
	5E 02			; ^ with NE pixel
	26 0C			; & with 
	5F 09			; _ with Gray Bar
	5B 10			; [ with (
	7B 10			; { with (
	5D 11			; ] with )
	7D 11			; } with )
	5C 18			; \ with /
	7C 05			; | with left black bar
	27 0B			; ' with "


