;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4084

@PushReg=0220			;Push reg then start Display-1
@Disp3=0292			;DISPLAY BLANK LINES routine 
@Video01=0281			;Video-1 routine
@PopReg=02A4			;Pop reg then return control to program
@Break=0F46			;Break-1 routine
@HRDFile=2000			;Hires DFile
@DFile=400C			;Address of display file
@LastK=4025			;Keyboard debounce value
@Frames=4034			;SYS VAR: Frames counter
@WorkRAM=8000			;Scratchpad area

;================================
; WRX Video driver by Wilf Rigter
; WRX16K - 1996
; Annotations by Wilf Rigter
;================================
@WRX
	LD IX,HR		;Simple start of the hres mode
	RET
@LBuf
	LD R,A			;Load HRDFile address LSB
	00 00 00 00		;32 NOPs = 256 pixels
	00 00 00 00 		;00=Black on white display
	00 00 00 00 		;80=White on black display
	00 00 00 00 
	00 00 00 00 
	00 00 00 00 
	00 00 00 00 
	00 00 00 00 
	RET NZ			;Always returns
@HR    
	LD B,06			;Delay
@HR0
	DJNZ HR0
	NOP			;More delay fine tuning
	NOP
	NOP
	NOP
	NOP
	DEC B
@HDAddr
	LD HL,HRDFile		;HRDFile
	LD DE,20		;32 bytes per line
	LD B,C0			;192 lines per hires screen
@HR1
	LD A,H			;Get HRDFile address MSB
	LD I,A			;Load MSB into I register
	LD A,L			;Get HRDFile address LSB
	CALL C089		;CALL Lbuf+8000
	ADD HL,DE		;Next line
	DEC B			;Dec line counter
	JP NZ HR1		;Last line
@HR2   
	CALL Disp3		;Return to application program
	CALL PushReg		;Extra register PUSH and VSYNC
               			;This code segment is optional
@Brake
	CALL Break		;Check break key
	LD A,1E			;Restore pattern table pointer
	LD I,A
	JR NC Stop		;Skip the HR vector load if BREAK
	LD IX,HR		;Load the HR vector
@Stop
	JP PopReg		;Return to application program
;================================
; Jump table
;================================
	JR WRX			;16608
	JR LoRes		;16610
	JR HRCls		;16612
	JR Invert		;16614
	JR Splash		;16616
	JR Transfer		;16618
;================================
; Restore lores display
;================================
@Lores
	HALT			;Wait for an interrupt - used to sync the changeover!
	LD A,(Frames) 		;Wait for the next new tv frame
	LD C,A
@Sync
	LD A,(Frames)
	CP C
	JR Z Sync
	LD A,1E		 	;Reset the "I" Register
	LD I,A		 	;to the ROM default (30 decimal)
	LD IX,Video01	 	;Pointer to the ROM display routine set!
	LD BC,0000
	RET
;================================
; Splash screen
;  Usage:
;   xx10 LOAD "SHOCK/SHOCK.BMP;8130"
;   xx20 LET L=USR Splash
;================================
@Splash
	CALL WRX		;Display Splash screen
	LD A,(LastK)
	CP FF
	RET NZ
	JR Splash+3
;================================
; WRX invert screen
;================================
@Invert
	LD HL,Lbuf+2		;Set source
	LD BC,2000		;Byte count, 32
	LD A,(HL) 		;Invert the value by adding 128
	ADD A,80
	LD (HL),A		;Set the inverted value 
	INC HL
	DJNZ Invert+6		;Repeat 32x
	RET			;Return 0 to BASIC
;================================
; Clear the HRDFile
;================================
@HRCls
	LD HL,HRDFile		;Set source
	LD (HL),00		;Set source to 0
	LD DE,HRDFile+1		;Set destination
	LD BC,17FF		;Byte count, HRDFile size-1
	LDIR			;Clear the HRDfile
	RET			;RET 0 to BASIC
;========================================
;COPY DFile contents to HRDFile
;========================================
@Transfer
	LD B,(IY+33)		;Get row count from (Spare1)
	LD C,00
	LD HL,HRDFile		;HRDFile location
	LD DE,(DFile)		;DFile location
	INC DE 			;Skip byte 118
@Loop1
	LD A,(DE) 		;Get DFile character
	CP 76	 		;EOL?
	JR Z NextLine		;Yes, start next line
	PUSH HL 		;PUSH HRDFile address
	PUSH DE 		;PUSH DFile address
	PUSH BC 		;PUSH row counter
	CP 40 			;Graphic chr?
	JR C Normal		;Jump if not graphic chr
	SBC A,40 		;Subtract 64 from character code
@Normal
	LD C,A
	XOR A			;Clear Carry 
	LD B,A
	LD DE,BBCFont+9		;Pattern table base address
	RL C 			;Code*2
	RL B
	RL C	 		;Code*4
	RL B 
	RL C 			;Code*8
	RL B 
	EX DE,HL	 	;HL=Pattern table
	ADD HL,BC 		;HL=Pattern pointer
	EX DE,HL 		;DE=Pattern pointer
	LD B,08 		;Byte counter
@NextByte
	LD A,(DE)	 	;Byte Pattern
	PUSH BC 		;PUSH bytes done counter
	LD (HL),A 		;Transfer bit pattern to HRDFile
	LD BC,0020	 	;Next row in HRDFile
	ADD HL,BC
	POP BC 			;POP bytes done counter 
	INC DE 			;Next bit pattern
	DJNZ NextByte		;Process 8 bytes
	POP BC 			;POP row counter
	POP DE 			;POP DFile address
	POP HL 			;POP HRDFile address
	INC DE 			;Next DFile address
	INC HL			;Next HRDFile address
	JR Loop1		;Copy next DFile character
@NextLine		
	INC C 			;Next HRDFile row
	INC DE 			;Next DFile address
	LD HL,HRDFile 
	PUSH BC 		;PUSH DFile rows done counter
	LD B,C 			;BC=Row offset to add to HRDFile base address
	LD C,00
	ADD HL,BC
	POP BC 			;POP DFile rows done counter
	DJNZ Loop1		;Do next DFile row
@BBCFont
	RET