;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 5497

@PushReg=0220			;Push reg then start Display-1
@Disp3=0292			;DISPLAY BLANK LINES routine 
@Video01=0281			;Video-1 routine
@PopReg=02A4			;Pop reg then return control to program
@Break=0F46			;Break-1 routine
@HRDFile=8100			;Hires DFile
@DFile=400C			;Address of display file
@LastK=4025			;Keyboard debounce value
@Frames=4034			;SYS VAR: Frames counter

;================================
; WRX Video driver by Wilf Rigter
; WRX16K - 1996
; Annotations by Wilf Rigter
;================================
@WRX
	LD IX,HR		;Simple start of the hres mode
	RET
@LBuf
	LD R,A			;Load HRDFile address LSB
	00 00 00 00		;32 NOPs = 256 pixels
	00 00 00 00 		;00=Black on white display
	00 00 00 00 		;80=White on black display
	00 00 00 00 
	00 00 00 00 
	00 00 00 00 
	00 00 00 00 
	00 00 00 00 
	RET NZ			;Always returns
@HR    
	LD B,06			;Delay
@HR0
	DJNZ HR0
	NOP			;More delay fine tuning
	NOP
	NOP
	NOP
	NOP
	DEC B
@HDAddr
	LD HL,HRDFile		;HRDFile
	LD DE,20		;32 bytes per line
	LD B,C0			;192 lines per hires screen
@HR1
	LD A,H			;Get HRDFile address MSB
	LD I,A			;Load MSB into I register
	LD A,L			;Get HRDFile address LSB
	CALL D49C		;CALL Lbuf+8000
	ADD HL,DE		;Next line
	DEC B			;Dec line counter
	JP NZ HR1		;Last line
@HR2   
	CALL Disp3		;Return to application program
	CALL PushReg		;Extra register PUSH and VSYNC
               			;This code segment is optional
@Brake
	CALL Break		;Check break key
	LD A,1E			;Restore pattern table pointer
	LD I,A
	JR NC Stop		;Skip the HR vector load if BREAK
	LD IX,HR		;Load the HR vector
@Stop
	JP PopReg		;Return to application program
;================================
; Jump table
;================================
	JR WRX			;21747
	JR LoRes		;21749
	JR HRCls		;21751
	JR Invert		;21752
	JR Screen		;21755
;================================
; Restore lores display
;================================
@Lores
	HALT			;Wait for an interrupt - used to sync the changeover!
	LD A,(Frames) 		;Wait for the next new tv frame
	LD C,A
@Sync
	LD A,(Frames)
	CP C
	JR Z Sync
	LD A,1E		 	;Reset the "I" Register
	LD I,A		 	;to the ROM default (30 decimal)
	LD IX,Video01	 	;Pointer to the ROM display routine set!
	LD BC,0000
	RET
;================================
; Splash screen
;  Usage:
;   xx10 LOAD "SHOCK/SHOCK.BMP;8130"
;   xx20 LET L=USR 21655
;================================
@Screen
	CALL WRX		;Display Splash screen
	LD A,(LastK)
	CP FF
	RET NZ
	JR Screen+3
;================================
; WRX invert screen
;================================
@Invert
	LD HL,Lbuf+2		;Set source
	LD BC,2000		;Byte count, 32
	LD A,(HL) 		;Invert the value by adding 128
	ADD A,80
	LD (HL),A		;Set the inverted value 
	INC HL
	DJNZ Invert+6		;Repeat 32x
	RET			;Return 0 to BASIC
;================================
; Clear the HRDFile
;================================
@HRCls
	LD HL,HRDFile		;Set source
	LD (HL),00		;Set source to 0
	LD DE,HRDFile+1		;Set destination
	LD BC,17FF		;Byte count, HRDFile size-1
	LDIR			;Clear the HRDfile
	RET			;RET 0 to BASIC
	NOP