;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4FD2			;20434

@PrntAt=08F5			;PRINT AT Y,X ROM routine
@DFile=400C			;Address of display file
@Vars=4010			;Start of Vars
@LastK=4025			;Keyboard debounce value
@Frames=4034			;16436, Frame counter
@Coords=4036			;16438, PLOT coords
@LoopFlag=4083			;16515, Player On/OFF flag
@LoadScr=4CC5			;19653, Load main form
@Scratch=BCE0			;Scratch pad area

;================================
; Jump Table
;================================
	JP RandLoop		;20434 Random Screen
	JP GoodLIFE		;20437 GoodLIFE
	JP Kaleid01		;20440 Kaleidoscope Style 1
	JP Kaleid02		;20443 Kaleidoscope Style 2
	JP Kaleid03		;20446 Pixel Kaleidoscope
	JP PxlMandala		;20449 Pixel Mandala
	JP ZXMandala		;20452 ZX81 Mandala
	JP Cityscape		;20455 Cityscape
	00 00 00
	00 00 00
@Density
	00
@BKG
	00
@Chr
	00
;================================
; Random screen loop
;================================
@RandLoop
	CALL SaveDF		;Save main form
	CALL SetPars
@Outer
	LD HL,Scratch		;Work area
	LD BC,0318		;Byte count
@Inner
	PUSH HL			;PUSH DFile address
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD L,A			;L=range 0 to 255
@Rand01
	LD A,(Density)		;Density value
	CP L			;L>Density?
@Rand02
	LD A,(Chr)		;Chr value
	JR C +3			;Print chr if CY
@Rand03
	LD A,(Bkg)			;Print bkg if not CY
	POP HL			;POP DFile address
	BIT 6,(HL)		;N/L?
	JR NZ +1		;Yes, skip print
	LD (HL),A		;Print chr or bkg
	INC HL			;Next DFile address
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ Inner		;Loop until BC=0
	CALL LoadDF
	CALL Stopped
	JR Outer
;================================
; Set density value range 87 to 150
;================================
@SetPars
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND A			;Test for 0 condition
	JR Z SetPars		;Jump if density=0
	LD (Density),A		;Set Density
	AND 01			;Odd or Even?
	LD A,80			;BKG="g "
	JR Z +1			;Jump if even
	XOR A			;BKG=" "
	LD (BKG),A		;Set BKG, " " or "g "
	LD B,A			;B=Bkg
@NewChr
	LD E,0C			;Set upper limit, 12
	CALL Modulus
	CP 0C			;Test A
	JR NZ +2		;Jump if A<12
	LD A,"O"		;Use "O" chr if A=12
	ADD A,B			;Range 0 to 12 or 128 to 140
	CP B			;BKG=Chr?
	JR Z NewChr		;Jump if BKG=Chr
	LD (Chr),A		;Set chr to print
@CLS
	LD A,B			;Set BKG chr
	LD D,A			;CLS with BKG chr
	LD HL,(DFile)
	LD BC,0318		;Byte count
	INC HL
	BIT 6,(HL)		;N/L?
	JR NZ +1		;Yes, skip it
	LD (HL),D		;Print BKG chr.
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ -11		;Loop until BC=0
	RET
;================================
@SaveDF
	LD HL,(DFile)
	INC HL
	LD DE,Scratch
	JR MoveDF
@LoadDF
	LD HL,Scratch
	LD DE,(DFile)
	INC DE
@MoveDF
	LD BC,0318
	LDIR
	RET
;================================
; GoodLIFE
;================================
@GoodLIFE			;Generate Random Gen 0
	CALL SaveDF		;Save main form
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD B,A
	BIT 0,A
	LD A,07			;Pixel cell
	JR Z +2
	LD A,34			;"O" cell
	LD (Cell1+1),A
	LD (Cell2+1),A
	LD (Cell3+1),A
	LD HL,(DFile)
	LD A,B
	AND 3F			;Range 0 TO 63
	ADD A,A			;Range 0 to 126 
	LD (LDensity+1),A	;Set density value
	LD BC,0318		;Byte count
@GenLoop
	PUSH HL			;PUSH DFile address
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD L,A			;L=range 0 to 255
@LDensity
	LD A,00			;Density value
	CP L			;L>Density?
	LD A,80			;Bkg chr
	JR C +2			;Print Bkg if NC
@Cell1
	LD A,00			;"gO", cell chr
	POP HL			;POP DFile address
	BIT 6,(HL)		;N/L?
	JR NZ +1		;Yes, skip print
	LD (HL),A		;Print cell or bkg
	INC HL			;Next DFile address
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ GenLoop		;Loop until BC=0
;================================
; Analyze screen
;================================
@NextGen
	LD HL,(DFile) 		;HL points to the top of the DFile
	LD DE,0021		;Bytes/row
	LD C,16			;22 rows
@NxtLine
	LD B,20			;32 cells per line
@Analyze
	LD A,50			;Initialize cell counter in A with 50h
	CP (HL)			;This part checks block of 9 neighbouring cells
	ADC A,D			;increments ">80h" neighbour counter in A
	INC HL		 	;NCell
	CP (HL)
	ADC A,D
	INC HL		 	;NECell
	CP (HL)
	ADC A,D
	ADD HL,DE		;ECell
	CP (HL)
	ADC A,D
	ADD HL,DE		;SECell
	CP (HL)
	ADC A,D
	DEC HL			;SCell
	CP (HL)
	ADC A,D
	DEC HL			;SWCell
	CP (HL)
	ADC A,D
	SBC HL,DE		;WCell
	CP (HL)
	ADC A,D
	INC HL			;Home cell
	CP (HL)
	JR NC +6		;If home cell is >56
	CP 55
	JR NZ +12
@Baby
	LD (HL),88		;Mark for birth
	CP 55
	JR Z +6
	CP 56
	JR Z +2
;Death
	LD (HL),08		;Mark as dying
;Done
	XOR A
	SBC HL,DE
	DJNZ Analyze
	INC HL		 	;Advance to the next line
	DEC C			;Dec line counter
	JR NZ NxtLine		;Last line?
				;Now DFile is Tested for dying
				;and birth cell marks
				;first execute dying cells
	LD DE,0880		;D=Dying, E=Dead (BKG)
@Mark
	LD HL,(DFile)		;Get DFile
	LD BC,03F6		;Cell Counter, 3*246
@NMark
	INC HL			;First cell
	LD A,(HL)		;Test cell
	CP D	   		;Is it a dying cell (CHR$ 08)?
	JR NZ +1  		;Skip if not dying
	LD (HL),E    		;Set as BKG if dying cell
;NoDie
	DEC C			;Decrease cell counter
	JR NZ NMark  		;Next cell
	DEC B	  		;Next cell
	JR NZ NMark  		;Next cell
				;The same routine is used for birth
@Cell2
	LD A,00			;now execute the birth cycle
	CP E	   		;did we already do the births?
	JR Z +5     		;If birth and death done then bye
@Cell3
	LD DE,8800   		;D=Birth, E=Cell
	JR Mark			;Go forth and deliver
;End
	LD C,21			;Byte count
	SBC HL,BC
	INC HL			;HL=Row 23 addr-1 (N/L chr of row 22)
	LD DE,(DFile)		;DE=Row 01 addr-1 (N/L start of DFile)
	LDIR			;Copy row 23 to row 01
	EX DE,HL		;Copy row 02 to row 24
				;HL=Row 02 addr-1 (N/L chr of row 01)
				;DE=Row 24 addr-1 (N/L chr of row 23)
	LD C,21			;Byte count
	LDIR
	CALL Stopped		;Loop until keypressed
	JP NextGen
;================================
; Kaleidoscope 01
;================================
@Kaleid01
	CALL SetPars		;Set Density, BKG & Chr parameters
@KLoop01
	LD E,0C			;Set upper limit, 12
	CALL Modulus
	LD B,A          	;B=Ycoord
	LD E,10			;Set upper limit, 16
	CALL Modulus
        LD C,A          	;C=Xcoord
	CALL PrintCell		;Get chr to print, (BKG) or (Cell)
	CALL PrintQuads		;Print chr to the four quadrants
	CALL Stopped		;Get key press
	JR KLoop01		;No, loop 
;================================
; Kaleidoscope 02
;================================
@Kaleid02
	CALL SetPars		;Set density range to 87 to 150
@KLoop02
	CALL PrintCell		;Get chr to print, (BKG) or (Cell)
	PUSH AF			;PUSH chr to print
	LD E,10			;Set upper limit, 16
	CALL Modulus
	LD B,A			;Set Ycoord
	CALL Modulus
	LD C,A			;Set Xcoord
	POP AF			;POP chr to print
	PUSH BC
	CALL PrintQuads		;Print to the 4 quadrants
	POP BC			;POP Y/Xcoords
	LD A,B			;Swap Y/Xcoords
	LD B,C
	LD C,A
	CALL PrintQuads		;Print to the 4 quadrants
	CALL Stopped		;Test keys
        JR KLoop02
;================================
; Kaleidoscope By S.H. McGurrin
; Syntax v5,n01 p.19
;================================
@Kaleid03
	CALL SetPars
@KLoop03
	LD E,3F			;Set upper limit
	CALL Modulus		;Get Xcoord, range=0 to 63
	LD C,A
	LD E,2F			;Set upper limit
	CALL Modulus		;Get Ycoord, range=0 to 47
	LD B,A			;BC=Y/Xcoords
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 01
        JR Z +2			;Jump if even
        LD A,40            	;40 FOR Plot, 0 FOR UNPlot
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT X,Y
        LD A,2F			;47
        SUB B
        LD B,A
	POP AF 		   	;POP Plot/UnPlot Flag
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT X,47-Y
        LD A,3F			;63
        SUB C
        LD C,A
	POP AF		   	;POP Plot/UnPlot Flag
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT 63-X,47-Y
        LD A,2F			;47
        SUB B
        LD B,A
	POP AF		 	;POP Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT 63-X,Y
	CALL Stopped
        JR KLoop03
;================================
; ZX81 Mandala
;================================
@ZXMandala
	CALL SetPars
	LD A,(BKG)		;Get BKG
	LD (MandSet+1),A	;Set BKG
	LD A,(Chr)		;Get chr
	LD (MandSet+5),A	;Set chr
	CALL Rnd8Bit		;Get random number
	AND 01
	JR NZ +4		;1, print cell chr
@MandSet
	LD A,00			;0, print BKG chr
	JR +2
	LD A,00			;Print cell chr
	LD (Setchr+1),A		;Set chr to print
	LD E,15			;Set upper limit
	CALL Modulus
	LD B,A			;B=0 to 21
	LD (Coords+1),A		;Save coord
	CALL Modulus
	LD C,A			;C=0 to 21
	LD (Coords),A		;Save coord
	LD A,08
	ADD A,C
	LD C,A
	CALL Print		;B,C+8
	LD E,C
	LD A,08
	ADD A,B
	LD C,A
	LD B,E
	CALL Print		;C,B+8
	LD E,C
	LD A,18
	SUB B
	LD C,A
	LD B,E
	CALL Print		;C,24-B
	LD A,18
	SUB C
	LD C,A
	CALL Print		;B,24-C
	LD A,17
	SUB B
	LD B,A
	LD A,08
	ADD A,C
	LD C,A
	CALL Print		;23-B,C+8
	LD E,B
	LD A,17
	SUB C
	LD B,A
	LD A,08
	ADD A,E
	LD C,A
	CALL Print		;23-C,B+8
	LD E,B
	LD A,17
	SUB C
	LD B,A
	LD A,18
	SUB E
	LD C,A
	CALL Print		;23-C,24-B
	LD A,17
	SUB B
	LD B,A
	LD A,18
	SUB C
	LD C,A
	CALL Print		;23-B,24-C
	CALL Stopped		;Check keys
	JR MandSet-7		;Loop until "Q" pressed
;================================
; Calculate print position then print chr
;  B=Ycoord
;  C=Xcoord
;================================
@Print
	LD HL,0000		;Set accumulator to 0
	LD DE,0021		;Bytes/row
	LD A,B
	AND A			;Row=0?
	JR Z +3			;Yes, move on
	ADD HL,DE		;HL=33*row value
	DJNZ -3
	ADD HL,BC		;HL=33*row value+column offset
	LD BC,(DFile)		;Get DFile
	INC BC			;Skip 1st N/L chr
	ADD HL,BC		;HL=print position
@Setchr
	LD (HL),00		;PRINT AT B,C;chr
	LD BC,(Coords)		;Restore coords
	RET
;================================
; Cityscape
;================================
@Cityscape
	CALL RND8Bit		;Get random number
	LD (Starz+1),A		;Set random number
	AND 0F			;Range 0 to 15
	LD B,A			;Ycoord, 0 to 15
	LD C,1F			;Xcoord, 31
	CALL PrntAt		;Set PRINT AT
	LD A,(HL)		;Get (HL) chr
	CP 80			;Building chr?
	JR NZ +17		;Skip star print if building chr
@Starz
	LD A,00			;Random number
	CP 20
	LD B,97			;Star chr "g*"
	JR C +8			;Print "g*" if 0 to 31
	CP 40
	LD B,95			;Star chr "g+"
	JR C +2			;Print "g+" if 32 to 63
	LD B,9B			;Star chr "g."
	LD (HL),B		;Otherwise print "g."
;ScrollLeft
	LD HL,(DFile)		;Get DFile
	INC HL			;Skip first N/L
	LD A,18			;Row counter
	LD B,00			;Clear B
	LD D,H			;Set destination
	LD E,L
	PUSH DE			;PUSH destination
	INC HL			;Set Source
	LD C,1F			;Byte count
	LDIR			;Shift row left
	POP HL			;POP destination
	LD C,21			;Bytes/row
	ADD HL,BC		;Next row
	DEC A			;Decrease row counter
	JR NZ -15		;Loop until A=0
;Build skyscraper
	CALL RND8Bit		;Get random number
	LD B,A			;Save random number
	BIT 0,A			;Odd or even?
	LD A,90			;Beacon right if even
	JR Z +1			;Jump if even
	INC A			;Beacon left if odd
	LD (Roof+4),A		;Set Beacon chr
	LD A,B			;Restore random number
	AND 07			;Range 0 to 7
	ADD A,04		;Range 4 to 11, building chr
	LD (Floor+3),A		;Set building chr
	LD E,0C			;Upper limit=12
	CALL Modulus		;A=0 to 12,
	INC A			;A=1 to 13=Building height
	LD B,A			;B=building height
	LD A,17			;23 bytes/column
	SUB B			;A-B=Sky value
	LD HL,(Vars)		;PRINT AT 23,31
	DEC HL
	DEC HL
	LD (HL),89		;Print sidewalk
	LD DE,0021		;33 bytes/row
@Floor
	SBC HL,DE		;Next floor up
	LD (HL),00		;Print "POKEd" floor chr
	DJNZ Floor		;Print B floors
	LD B,A			;B=Sky count
	CP 0A			;Print beacon if A=10
	JR Z Roof		;Print roof chr
	CP 0B			;Print blinker if A=11
	JR NZ +4
	LD A,81			;Blinker chr
	JR +6
	CP 0C			;Print flasher if A=12
	JR NZ +10
	LD A,99			;Flasher chr
	LD (Roof+4),A		;Set rooftop chr
@Roof
	DEC B			;Decrease sky count
	SBC HL,DE		;Roof print position
	LD (HL),00		;Print "POKEd" rooftop chr
;Sky
	LD A,80			;Sky chr
;SkyLoop
	SBC HL,DE		;Sky print position
	LD (HL),A		;Print sky
	DJNZ -5			;Print B sky chrs
@RoofTop
	LD HL,(DFILE)
	LD BC,0319
	LD A,(HL)
	CP 80			;Skip these chrs
	JR Z RoofPrnt+1
	CP 76
	JR Z RoofPrnt+1
	BIT 7,A
	JR Z RoofPrnt+1
;Beacon animation
	LD DE,91B4
	CP D
	JR Z RoofPrnt
	LD DE,B490
	CP D
	JR Z RoofPrnt
	LD DE,9091
	CP D
	JR Z RoofPrnt
;Blinker animation
	LD DE,8182
	CP D
	JR Z RoofPrnt
	INC D
	DEC E
	CP D
	JR Z RoofPrnt
;Flasher animation
	LD DE,999A
	CP D
	JR Z RoofPrnt
	INC D
	DEC E
	CP D
	JR NZ +1
@RoofPrnt
	LD (HL),E
	INC HL
	DEC BC 
	LD A,B 
	OR C 
	JR NZ RoofTop+6
	CALL Stopped		;Check keys
	JP CityScape		;Build next building
;================================
; 64x48 Plot Routine
;
;  B=YCoord, 0 to 43
;  C=XCoord, 0 to 63
;  A=40H/64=PLOT
;  A=00=UNPLOT
;===============================
@Plot
	PUSH BC			;PUSH X,Y coords
	PUSH AF			;PUSH Plot/UnPlot flag
	LD A,87			;A=87=NW Pixel
	BIT 0,B			;Y coord
	JR NZ +2		;Odd line number?
	ADD A,10		;A=97=SW Pixel 
	BIT 0,C			;X coord
	JR Z +2			;Even col number?
	ADD A,08		;A=8F=NE Pixel or A=9F=SE Pixel
	LD D,A			;D=Pixel quadrant value
	POP AF			;POP Plot/UnPlot flag
	ADD A,D			;A=D IF UNPlot or D+64 IF Plot
	LD (PlotPX+1),A		;Self modifying code
	SRL B			;Convert Y-coord to Print coord, Y/2
	SRL C			;Convert X-coord to Print coord, X/2
	LD HL,(DFile)		;Get DFile
	INC HL			;Start of DFile
	LD D,00
	LD E,C			;DE=Column number
	ADD HL,DE		;HL=Column position in DFile
	LD A,17			;23-Y/2
	SUB B
	LD B,A			;B=Row number
	JR Z +6
	LD DE,0021		;Bytes/row
	ADD HL,DE
	DJNZ -3
	LD A,(HL)		;Get chr at DFile addr
	CP 08			;LO Pixel Chr
	JR C PlotPX
	CPL			;HI Pixel Chr
@PlotPX				
	SET 00,A		;00=SR SET/RES X,A
	BIT 3,A
	JR Z +5
	CPL
	AND 8F
	JR +2
	AND 0F			;Low value PIXEL, codes 1 to 7
	LD (HL),A		;Hi value PIXEL, codes 128 to 135
	POP BC			;POP X,Y coords
	RET
;================================
; Pixel Mandala
;================================
@PxlMandala
	CALL SetPars		;CLS with "g " or " "
	BIT 7,D			;Test CLS chr
	JR Z Mandala		;Skip next section if D was 0
	LD BC,0020		;Print last line
	SBC HL,BC
	DEC C
	LD (HL),03		;Pixel line chr
	LD D,H
	LD E,L
	INC DE
	LDIR
@Mandala
	LD A,40			;Plot flag
	LD (MandPlot+1),A	;Set for plot
	LD B,02 
@MandLoop
	PUSH BC 		;PUSH counter
	LD E,2B			;Set upper limit to 45
	CALL Modulus		;Get Xcoord, range 0 to 45
	LD D,A			;Xcoord
	PUSH DE 		;PUSH Xcoord
	CALL Modulus		;Get Ycoord, range 0 to 45
	POP DE 			;POP Xcoord
	LD E,A 			;DE=XYcoords
	LD (Coords),DE		;Store coords
	CALL QuadPlot		;Plot quads
	XOR A			;Unplot flag
	LD (MandPlot+1),A	;Set for unplot
	POP BC 			;POP counter
	DJNZ MandLoop		;Loop back
	CALL Stopped
	JR Mandala
;================================
; Mandala Plot Quads
;  DE=Initial coords
;================================
@QuadPlot
	LD A,0C
	ADD A,D
	LD C,A
	LD B,E
	CALL MandPlot		;D+12,E
	LD A,0C
	ADD A,E
	LD C,A
	LD B,D
	CALL MandPlot		;E+12,D
	LD A,0C
	ADD A,E 
	LD C,A
	LD A,28 
	SUB D 
	LD B,A
	CALL MandPlot		;E+12,40-D
	LD A,0C 
	ADD A,D
	LD C,A
	LD A,28 
	SUB E 
	LD B,A
	CALL MandPlot		;D+12,40-E
	LD A,34 
	SUB D 
	LD C,A
	LD B,E
	CALL MandPlot		;52-D,E
	LD A,34 
	SUB E 
	LD C,A
	LD B,D
	CALL MandPlot		;52-E,D
	LD A,34 
	SUB E 
	LD C,A
	LD A,28 
	SUB D 			
	LD B,A
	CALL MandPlot		;52-E,40-D
	LD A,34 
	SUB D 
	LD C,A
	LD A,28 
	SUB E 
	LD B,A
	CALL MandPlot		;52-D,40-E
	RET
;================================
; PLOT Mandala Quads
;================================
@MandPlot
	LD A,00			;PLOT flag
	INC B			;Increase Ycoord before plotting
	INC B
	INC B
	INC B
	CALL Plot
	LD DE,(Coords)	 	;Restore coords
	RET
;================================
; Set print chr to (BKG) or (Cell)
; based on random number generated.
;================================
@PrintCell
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD L,A			;L=8-Bit random number, range 0 to 255
	LD A,(Density)		;Get density value
	CP L			;L>Density?
	LD A,(Chr)		;Get chr to print
	JR C +3			;Yes, print cell
	LD A,(BKG)		;No, print bkg
	LD (PokeA+1),A		;POKE chr to print
	RET
;================================
; Print A to the four quadrants
; of the screen.
;================================
@PrintQuads
	CALL PrintX		;Print to NW quadrant B,C
	LD A,1F
        SUB C
        LD C,A
	CALL PrintX		;Print to NE quadrant B,31-C
	LD A,17
        SUB B
        LD B,A
	CALL PrintX		;Print to SE quadrant 25-B,31-C
	LD A,1F			
        SUB C
	LD C,A
;================================
; Locate print position in DFile
; then print POKEd chr.
;================================
@PrintX
	PUSH DE
	PUSH BC
	INC B			;Ycoord must be > 0
	LD HL,(DFile)		;Get start of DFile
	INC HL
	LD DE,0000		;Initialize DE
	ADD HL,DE		
	LD E,21			;DE=33 bytes/row
	DJNZ -5			;Add Bx33 to start of DFile
	LD E,C			;DE=Xcoord offset
	ADD HL,DE		;Add X offset to DE
@PokeA
	LD (HL),00		;POKEd chr to print
	POP BC			;POP Y/Xcoords
	POP DE
	RET
;================================
; Wait for key release
;================================
@Release
	LD A,(LastK)		;Check for keypress
	INC A
	JR NZ Release		;Loop until key released
	RET
;================================
; If tune ended or keypressed then
; return to BASIC
;================================
@Stopped			;First test stopped bit
	LD HL,LoopFlag		;Loopflag addr
	BIT 7,(HL)		;Test stopped bit.
	JR Z KeyTest		;If 0 move on, still playing
	RES 7,(HL)		;Tune ended, reset stopped flag
	BIT 0,(HL)		;Test loop flag
	JR NZ TuneDone		;If NZ end video
@KeyTest			;Test for keypress
	LD B,10
        LD C,01 		;Delay Loop
	DEC BC 			;Delay
	LD A,B 
	OR C 
	JR NZ -5
	LD A,(LastK+1)		;Check for keypress
	INC A
	RET Z			;Return if no keypress
	LD HL,Keytest+1
	CP FC			;"S"lower
	JR NZ +2
	INC (HL)
	RET
	CP F0			;"F"aster
	JR NZ +4
	DEC (HL)
	RET P			;Return if Positive
	INC (HL)		;Minimum delay value for B is 00
	RET
	POP BC			;Clear stack
	JR Release		;Return via Release
@TuneDone
	CALL LoadScr		;Restore screen
	POP BC			;Clear the stack
	CALL Release		;Wait for key release
	LD BC,+90		;
	RET			;GOTO 90
;================================
; 8-Bit Random Number Generator
; Reg BC not used
; On exit...
;  Reg A=random number in the
;        range 0 to 255
;================================
@Rnd8Bit
	LD DE,0000		;Seed
	LD HL,(Frames)     	;Create pointer to ROM address
	DEC DE	
	ADD HL,DE
	LD A,H
	AND 1F
	LD H,A			;Limit address to <=8192
	LD (Rnd8Bit+1),HL	;New seed
	LD A,(HL)		;Get value at ROM address
	RET
;================================
; Modulus code to ensure 
; number in desired range.
; On entry...
;  E=Upper Limit
; Reg BC not used
; On exit...
;  A=random number between 0 and
;    upper limit
;================================
@Modulus
	LD D,00
	INC E			;DE=dividend
	PUSH DE
	CALL Rnd8Bit		;Get 8-Bit random number
	POP DE
	LD H,D			
	LD L,A			;HL=8-Bit random number
@ModLoop
	OR A			;Clear CY
	SBC HL,DE		;Subtract dividend from HL
	JP P ModLoop		;Keep doing it till negative
	ADD HL,DE		;Add the divided back again
	LD A,L			;Return results in A
	DEC E
	RET